/*
 * Copyright  1995, interface business GmbH, Dresden.
 *      All rights reserved.
 *
 * This code has been written by Jrg Wunsch, Dresden.
 * Direct comments to <joerg_wunsch@interface-business.de>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *      $Id: main.c,v 1.3 1995/10/30 14:20:06 j Exp $
 */

#include "upsmon.h"

static const char *progname;
static time_t last_selftest = 0;
static int battery_low = 0;
static struct bargraph *line, *batt, *load, *fill;

#define STDIN 0
#define DEFAULT_TIMEOUT 10

static void
usage(void)
{
  fprintf(stderr, "usage: %s [-t timeout] ttyname\n", progname);
  exit(2);
}

static void
handle_ups_event(void)
{
  char c;
  if(read(ups_fd, &c, 1) < 1)
    return;
  
  switch(c)
    {
    case UPS_EV_TURNON:
      last_selftest = time(0);
      (void)ups_command(UPS_CMD_SMART);
      break;
      
    case UPS_EV_POWER:
      if(battery_low)
	message("Input line power fail, battery low");
      else
	message("Input line power fail");
      break;

    case UPS_EV_PWRBACK:
      message("Input line power back");
      break;

    case UPS_EV_BATLOW:
      /* don't warn if this is immediately after a selftest/turnon */
      if(time(0) - last_selftest < 5)
	break;
      battery_low = 1;
      message("Battery low");
      break;

    case UPS_EV_BATUP:
      battery_low = 0;
      last_selftest = 0;
      message("Battery up again");
      break;
    }
}

static void
disp_setup(void)
{
  char b[1024], c[20];
  /* NB: can't use sprintf() here, ups_command uses static storage */
  strcpy(b, "Model:                 ");  strcat(b, ups_command(UPS_CMD_MODEL));
  strcat(b, " ");                 strcat(b, ups_command(UPS_CMD_COPYRIGHT));
  strcat(b, "\n");
  strcat(b, "Manufact. date:        ");  strcat(b, ups_command(UPS_CMD_MDATE));
  strcat(b, "\n");
  strcat(b, "Serial #:	            ");  strcat(b, ups_command(UPS_CMD_ID));
  strcat(b, "\n");
  strcat(b, "Local ID:              ");strcat(b, ups_command(UPS_CMD_LOCALID));
  strcat(b, "\n");
  strcat(b, "Nominal line voltage:  ");  strcat(b, ups_command(UPS_CMD_NLINE));
  strcat(b, " V\n");
  strcat(b, "Nominal batt. voltage: ");
  sprintf(c, "%ld V", ups_read_int(UPS_CMD_NBATTERY));
  strcat(b, c);
  strcat(b, "\n\n(press any key)");

  messagebox(12, 60, 5, 10, b, 0);
}

  
static int
handle_kbd_event(void)
{
  int c;

  c = getchar();
  switch(c)
    {
    case 'q':
    case 'Q':
      return 1;

    case 's':
    case 'S':
      disp_setup();
      break;
      
    case 'r':
    case 'R':
    case 'l'&0x1f:
      wrefresh(stdscr);
      break;
    }
  
  return 0;
}

static void
handle_statistics(void)
{
  disp_status(ups_read_stat());
  update_bargraph(line, ups_read_double(UPS_CMD_LINE));
  update_bargraph(batt, ups_read_double(UPS_CMD_BATTERY));
  update_bargraph(load, ups_read_double(UPS_CMD_POWER));
  update_bargraph(fill, ups_read_double(UPS_CMD_FILL));
}


void
quit(int signo)
{
  ups_reset();
  
  move(LINES-1, 0);
  refresh();
  endwin();
  putchar('\n');
  exit(0);
}

int
main(int argc, char **argv)
{
  int i, errs = 0, terminate = 0;
  char *cp, *cp1, *cp2;
  fd_set ifds;
  struct timeval tmo;
  long timeout = DEFAULT_TIMEOUT;

  if((cp = strrchr(argv[0], '/')))
    progname = cp + 1;
  else
    progname = argv[0];
  
  while((i = getopt(argc, argv, "t:")) != EOF)
    switch(i)
      {
      case 't':
	timeout = atol(optarg);
	if(timeout <= 0)
	  {
	    fprintf(stderr, "%s: invalid timeout %ld, ignored\n",
		    progname, timeout);
	    timeout = DEFAULT_TIMEOUT;
	  }
	break;
	
      case '?':
	errs++;
      }
  argc -= optind;
  argv += optind;

  if(errs || argc != 1)
    usage();
  
  ups_init(argv[0]);
  cp = strdup(ups_command(UPS_CMD_MODEL));
  cp1 = strdup(ups_command(UPS_CMD_LOCALID));
  cp2 = (char *)malloc((strlen(cp)+strlen(cp1)+5)*sizeof(char));
  if(cp && cp1 && cp2)
    sprintf(cp2, "%s (%s)", cp, cp1);
  terminal_init(cp2);
  free(cp); free(cp1); free(cp2);
  signal(SIGINT, quit);
  line = init_bargraph(5, COLS/2-2, 3, 0,
		       200.0, 250.0, 10.0, "%3.0f",
		       " Line voltage, V ",
		       7, "%5.1f");
  batt = init_bargraph(5, COLS/2-2, 10, 0,
		       20.0, 30.0, 2.0, "%2.0f",
		       " Battery voltage, V ",
		       7, "%5.2f");
  load = init_bargraph(5, COLS/2-2, 3, COLS/2,
		       0.0, 100.0, 20.0, "%2.0f",
		       " Power load, % ",
		       7, "%5.1f");
  fill = init_bargraph(5, COLS/2-2, 10, COLS/2,
		       0.0, 100.0, 20.0, "%2.0f",
		       " Battery level, % ",
		       7, "%5.1f");

  do
    {
      handle_statistics();
      FD_ZERO(&ifds);
      FD_SET(ups_fd, &ifds);
      FD_SET(STDIN, &ifds);
      tmo.tv_sec = timeout;
      tmo.tv_usec = 0;
      if(select(FD_SETSIZE, &ifds, 0, 0, &tmo) < 0)
	continue;		/* `break' causes trouble for ^Z/fg */
      if(FD_ISSET(ups_fd, &ifds))
	handle_ups_event();
      if(FD_ISSET(STDIN, &ifds))
	terminate = handle_kbd_event();
    }
  while(!terminate);

  quit(0);
  return 99;
}
