/*
 * Copyright  1995, interface business GmbH, Dresden.
 *      All rights reserved.
 *
 * This code has been written by Jrg Wunsch, Dresden.
 * Direct comments to <joerg_wunsch@interface-business.de>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *      $Id: terminal.c,v 1.3 1995/10/30 14:20:09 j Exp $
 */


#include "upsmon.h"

static WINDOW *msgline, *msgframe;
static WINDOW *statline, *statframe;

void
terminal_init(const char *headline)
{
  initscr();			/* Always call initscr() first */
  if(LINES < 24 || COLS < 80)
    {
      fprintf(stderr,
	      "Sorry, your terminal is too small to run this program.\n"
	      "I need at least 24 * 80 characters\n");
      exit(1);
    }
  crmode();			/* We want cbreak mode */
  noecho();			/* We want to have control of chars */

  if(SO == 0 || *SO == 0)
    messagebox(6, 40, LINES/2-3, COLS/2-20,
	       "Your terminal does not support\n"
	       "standout mode; bargraph displays\n"
	       "won't look useful.  Press a key.",
	       0);

  if(headline || strlen(headline) > 0 || strlen(headline) < COLS)
    {
      move(0, (COLS-strlen(headline))/2);
      addstr(headline);
    }
  msgframe = subwin(stdscr, 3, COLS, LINES-4, 0);
  box(msgframe, '|', '-');
  wmove(msgframe, 0, 0); waddch(msgframe, '+');
  wmove(msgframe, 3-1, 0); waddch(msgframe, '+');
  wmove(msgframe, 0, COLS-1); waddch(msgframe, '+');
  wmove(msgframe, 3-1, COLS-1); waddch(msgframe, '+');
  wmove(msgframe, 0, 3); waddstr(msgframe, " Message line: ");
  wrefresh(msgframe);
  msgline = subwin(msgframe, 1, COLS-2, LINES-3, 1);
  
  statframe = subwin(stdscr, 3, 35, LINES-8, 0);
  box(statframe, '|', '-');
  wmove(statframe, 0, 0); waddch(statframe, '+');
  wmove(statframe, 3-1, 0); waddch(statframe, '+');
  wmove(statframe, 0, 35-1); waddch(statframe, '+');
  wmove(statframe, 3-1, 35-1); waddch(statframe, '+');
  wmove(statframe, 0, 3); waddstr(statframe, " UPS status: ");
  wrefresh(statframe);
  statline = subwin(statframe, 1, 35-2, LINES-7, 1);
}


/*
 * Popup a message box and display the message.  There are actually
 * two windows, one containing the frame only.
 *
 * If sleep is 0, wait until a key is pressed, otherwise sleep for
 * `sleep' seconds.
 */
 
void
messagebox(int height, int width, int topy, int topx,
	   const char *msg, int sleept)
{
  WINDOW *outer, *inner;

  outer = newwin(height, width, topy, topx);
  box(outer, '|', '-');
  wmove(outer, 0, 0); waddch(outer, '+');
  wmove(outer, height-1, 0); waddch(outer, '+');
  wmove(outer, 0, width-1); waddch(outer, '+');
  wmove(outer, height-1, width-1); waddch(outer, '+');
  wrefresh(outer);
  inner = subwin(outer, height-2, width-4, topy+1, topx+2);
  wmove(inner, 0, 0);
  waddstr(inner, msg);
  wrefresh(inner);
  move(LINES-1, 0);
  refresh();
  if(sleept)
    sleep(sleept);
  else
    (void)getchar();
  wclear(inner);
  wclear(outer);
  wrefresh(inner);
  wrefresh(outer);
  delwin(inner);
  delwin(outer);
  touchwin(stdscr);
  wrefresh(stdscr);
}

struct bargraph *
init_bargraph(int height, int width, int topy, int topx,
	      double min, double max, double step, const char *fmt,
	      const char *headline,
	      int digits, const char *fmt2)
{
  struct bargraph *bp;
  int i;
  double d;
  
  if((bp = (struct bargraph *)calloc(1, sizeof(struct bargraph))) == 0)
    abort();
  bp->win = subwin(stdscr, height, width, topy, topx);
  box(bp->win, '|', '-');
  wmove(bp->win, 0, 0); waddch(bp->win, '+');
  wmove(bp->win, height-1, 0); waddch(bp->win, '+');
  wmove(bp->win, 0, width-1); waddch(bp->win, '+');
  wmove(bp->win, height-1, width-1); waddch(bp->win, '+');
  wmove(bp->win, 1, 2);

  bp->max = max;
  bp->min = min;
  bp->len = width - 4;
  if(digits) bp->len -= digits + 2;
  bp->digits = digits;
  bp->fmt = fmt2;
  bp->unit = (max - min) / (double)bp->len;

  wstandout(bp->win);
  waddch(bp->win, ' ');
  wstandend(bp->win);
  
  if(height > 3)
    {
      /* draw a scale */
      if(digits)
	/* there's enough space to draw the rightmost value as well */
	max += step;
      for(d = min; d < max; d += step)
	{
	  if(height > 4)
	    {
	      wmove(bp->win, height-3, (d - min) / bp->unit + 2);
	      waddch(bp->win, '|');
	    }
	  wmove(bp->win, height-2, (d - min) / bp->unit + 2);
	  wprintw(bp->win, fmt, d);
	}
    }

  if(headline)
    {
      i = (width - strlen(headline)) / 2;
      if(i < 0) i = 0;
      wmove(bp->win, 0, i);
      waddstr(bp->win, headline);
    }

  if(digits)
    {
      wmove(bp->win, 1, bp->len + 2 + 2);
      wprintw(bp->win, fmt2, min);
    }
      
  wrefresh(bp->win);
  move(LINES-1, 0);
  refresh();
  
  return bp;
}

void
update_bargraph(struct bargraph *bp, double volume)
{
  int i, vol;
  double v2 = volume;
  
  if(v2 < bp->min) v2 = bp->min;
  if(v2 >= bp->max) v2 = bp->max;
  vol = (v2 - bp->min) / bp->unit;
  
  wmove(bp->win, 1, 2);
  wstandout(bp->win);
  for(i = 0; i < vol; i++) waddch(bp->win, ' ');
  wstandend(bp->win);
  for(/* i = i */; i < bp->len; i++) waddch(bp->win, ' ');
  if(bp->digits)
    {
      wmove(bp->win, 1, bp->len + 2 + 2);
      wprintw(bp->win, bp->fmt, volume);
    }
  wrefresh(bp->win);
  move(LINES-1, 0);
  refresh();
}

void
del_bargraph(struct bargraph *bp)
{
  delwin(bp->win);
  free(bp);
}


void
message(const char *msg)
{
  wmove(msgline, 0, 1);
  wclrtoeol(msgline);
  waddstr(msgline, msg);
  wrefresh(msgline);
  move(LINES-1, 0);
  refresh();
}


void
disp_status(struct ups_status s)
{
  wmove(statline, 0, 1);
  wclrtoeol(statline);

  if(s.not_avail)
    waddstr(statline, " *** not available *** ");
  else
    {
      if(s.line_pwr_fail)
	{
	  wstandout(statline);
	  waddstr(statline, " Power fail ");
	  wstandend(statline);
	}
      else
	waddstr(statline, " Line power");
      wmove(statline, 0, 13);
      if(s.standby)
	{
	  wstandout(statline);
	  waddstr(statline, " Standby ");
	  wstandend(statline);
	}
      else
	waddstr(statline, " On-line");
      if(s.battery_low)
	{
	  wmove(statline, 0, 22);
	  wstandout(statline);
	  waddstr(statline, " Batt.low ");
	  wstandend(statline);
	}
    }
  wrefresh(statline);
  move(LINES-1, 0);
  refresh();
}
