/*
 * Copyright  1995, interface business GmbH, Dresden.
 *      All rights reserved.
 *
 * This code has been written by Jrg Wunsch, Dresden.
 * Direct comments to <joerg_wunsch@interface-business.de>.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *      $Id: ups.c,v 1.3 1995/10/30 14:20:11 j Exp $
 */

#include "upsmon.h"

#define UPS_RESPONSE 256
static char ups_response[UPS_RESPONSE];

static struct termios omode, upsmode;

void
ups_init(const char *ttyname)
{
  if((ups_fd = open(ttyname, O_RDWR|O_EXCL|O_NONBLOCK, 0)) == -1)
    {
      perror("open(ups_tty)");
      exit(1);
    }
  /* turn on raw mode on UPS tty line */
  if(tcgetattr(ups_fd, &omode) == -1)
    {
      perror("tcgetattr(ups_tty)");
      exit(1);
    }
  upsmode = omode;
  cfmakeraw(&upsmode);
  (void)cfsetispeed(&upsmode, B2400);
  (void)cfsetospeed(&upsmode, B2400);  
  upsmode.c_cflag |= CLOCAL;
  if(tcsetattr(ups_fd, TCSANOW, &upsmode) == -1)
    {
      perror("tcsetattr(ups_tty, RAW)");
      exit(1);
    }
  /* turn UPS into smart mode */
  (void)ups_command(UPS_CMD_SMART);
}


void
ups_reset(void)
{
  /* turn UPS into dumb mode */
  (void)ups_command(UPS_CMD_DUMB);
  /* tty line back to normal mode */
  (void)tcsetattr(ups_fd, TCSANOW, &omode);
}

/*
 * Send a single-character command to the UPS.
 *
 * Try to read a response string from the UPS in return, up to the
 * terminating CR-LF sequence.  Store the result in a static buffer
 * (ignoring CR and LF), and return that buffer to the caller.
 */

char *
ups_command(int cmd)
{
  int i;
  char c;
  fd_set ifds;
  struct timeval tmo;
  
  ups_response[0] = 0;
  c = cmd;
  if(write(ups_fd, &c, 1) < 1)
    return ups_response;

  for(i = 0; i < UPS_RESPONSE - 1; i++)
    {
      FD_ZERO(&ifds);
      FD_SET(ups_fd, &ifds);
      tmo.tv_sec = 0;
      tmo.tv_usec = 100 * 1000;
      if(select(FD_SETSIZE, &ifds, 0, 0, &tmo) < 1)
	break;
      if(read(ups_fd, &c, 1) < 1)
	break;
      if(c == '\r')
	{
	  i--;
	  continue;
	}
      if(c == '\n')
	break;
      ups_response[i] = c;
    }
  ups_response[i] = 0;
  return ups_response;
}


struct ups_status
ups_read_stat(void)
{
  struct ups_status s;
  int st;
  char *cp;
  
  s.standby = s.line_pwr_fail = s.battery_low = 0;
  s.not_avail = 1;

  cp = ups_command(UPS_CMD_STATUS);
  if(cp == 0 || *cp == 0)
    return s;

  s.not_avail = 0;
  sscanf(cp, "%2x", &st);
  if(st & UPS_STATUS_BLOW)
    s.battery_low = 1;
  if(st & UPS_STATUS_ON)
    return s;
  if(st & UPS_STATUS_PWRF)
    {
      s.line_pwr_fail = 1;
      return s;
    }
  if(st & UPS_STATUS_STBY)
    {
      s.line_pwr_fail = s.standby = 1;
      return s;
    }
  s.standby = 1;
  return s;
}

double
ups_read_double(int cmd)
{
  char *cp;

  cp = ups_command(cmd);
  if(cp == 0 || *cp == 0 || strcmp(cp, UPS_RES_NA) == 0)
    return 0.0;

  return atof(cp);
}

long
ups_read_int(int cmd)
{
  char *cp;

  cp = ups_command(cmd);
  if(cp == 0 || *cp == 0 || strcmp(cp, UPS_RES_NA) == 0)
    return 0;

  return atol(cp);
}
